@@ -4,7 +4,7 @@
 #' @name predict.CauseSpecificCox
 #' @aliases predict.CauseSpecificCox
 #' @aliases predictBig.CauseSpecificCox
-#'
+#' 
 #' @param object The fitted cause specific Cox model
 #' @param newdata [data.frame or data.table]  Contain the values of the predictor variables
 #' defining subject specific predictions relative to each cause.
@@ -16,8 +16,8 @@
 #' @param type [character] Can be changed to \code{"survival"} if the event free survival should be output instead of the absolute risk.
 #' @param landmark [integer] The starting time for the computation of the cumulative risk.
 #' @param keep.times [logical] If \code{TRUE} add the evaluation times to the output.
-#' @param keep.newdata [logical] If \code{TRUE} add the value of the covariates used to make the prediction in the output list.
-#' @param keep.strata [logical] If \code{TRUE} add the value of the strata used to make the prediction in the output list.
+#' @param keep.newdata [logical] If \code{TRUE} add the value of the covariates used to make the prediction in the output list. 
+#' @param keep.strata [logical] If \code{TRUE} add the value of the strata used to make the prediction in the output list. 
 #' @param se [logical] If \code{TRUE} compute and add the standard errors to the output.
 #' @param band [logical] If \code{TRUE} compute and add the quantiles for the confidence bands to the output.
 #' @param iid [logical] If \code{TRUE} compute and add the influence function to the output.
@@ -27,38 +27,38 @@
 #' @param product.limit [logical]. If \code{TRUE} the survival is computed using the product limit estimator.
 #' Otherwise the exponential approximation is used (i.e. exp(-cumulative hazard)).
 #' @param store.iid [character] Implementation used to estimate the influence function and the standard error.
-#' Can be \code{"full"} or \code{"minimal"}.
+#' Can be \code{"full"} or \code{"minimal"}. 
 #' @param diag [logical] when \code{FALSE} the absolute risk/survival for all observations at all times is computed,
 #' otherwise it is only computed for the i-th observation at the i-th time.
 #' @param max.time [numeric] maximum time of the response of the fitted data.  Only relevant if
 #' model \code{response} element has been removed
 #' @param ... not used.
-#'
+#' 
 #' @author Brice Ozenne broz@@sund.ku.dk, Thomas A. Gerds
 #'     tag@@biostat.ku.dk
-#'
+#' 
 #' @details
 #' This function computes the absolute risk as given by formula 2 of (Ozenne et al., 2017).
 #' Confidence intervals and confidence bands can be computed using a first order von Mises expansion.
 #' See the section "Construction of the confidence intervals" in (Ozenne et al., 2017).
-#'
+#' 
 #' A detailed explanation about the meaning of the argument \code{store.iid} can be found
 #' in (Ozenne et al., 2017) Appendix B "Saving the influence functions".
-#'
+#' 
 #' Note: for Cox regression models with time varying
 #'     covariates it does not make sense to use this function, because
 #'     the predicted risk has to be a measurable function of the data
 #'     available at the time origin.
-#'
+#' 
 #' The iid decomposition is output using an array containing the value of the influence
 #' of each subject used to fit the object (dim 1),
 #' for each subject in newdata (dim 3),
 #' and each time (dim 2).
-#'
+#' 
 #' @seealso
 #' \code{\link{confint.predictCSC}} to compute confidence intervals/bands.
 #' \code{\link{autoplot.predictCSC}} to display the predictions.
-#'
+#' 
 #' @references
 #' Brice Ozenne, Anne Lyngholm Sorensen, Thomas Scheike, Christian Torp-Pedersen and Thomas Alexander Gerds.
 #' riskRegression: Predicting the Risk of an Event using Cox Regression Models.
@@ -94,11 +94,11 @@
 #' CSC.fit.s <- CSC(list(Hist(time,event)~ strata(X1)+X2+X9,
 #'  Hist(time,event)~ X2+strata(X4)+X8+X7),data=d, method = "breslow")
 #' predict(CSC.fit.s,cause=1,times=ttt,se=1L) ## note: absRisk>1 due to small number of observations
-#'
+#' 
 #' ## using the cph function instead of coxph
-#' CSC.cph <- CSC(Hist(time,event)~ X1+X2,data=d, method = "breslow", fitter = "cph")#'
+#' CSC.cph <- CSC(Hist(time,event)~ X1+X2,data=d, method = "breslow", fitter = "cph")#' 
 #' predict(CSC.cph, newdata = d, cause = 2, times = ttt)
-#'
+#' 
 #' ## landmark analysis
 #' T0 <- 1
 #' predCSC.afterT0 <- predict(CSC.fit, newdata = d, cause = 2, times = ttt[ttt>T0], landmark = T0)
@@ -156,41 +156,41 @@ predict.CauseSpecificCox <- function(object,
     ## ** prepare
     
     n.times <- length(times)
-    if(object$fitter=="phreg"){newdata$entry <- 0}
+    if(object$fitter=="phreg"){newdata$entry <- 0} 
     new.n <- NROW(newdata)
     ## if(data.table::is.data.table(newdata)){
     ## newdata <- data.table::copy(newdata)
     ## }else{
     ## newdata <- data.table::as.data.table(newdata)
     ## }
-
+    
     surv.type <- object$surv.type
     if (missing(cause)) {
         cause <- object$theCause
     }
     if (length(cause) > 1){
-        stop(paste0("Can only predict one cause. Provided are: ",
+        stop(paste0("Can only predict one cause. Provided are: ", 
                     paste(cause, collapse = ", "), sep = ""))
     }
-
+	
     ## causes
     # NOTE: cannot use only eventtimes of cause 1 otherwise wrong estimation of the survival in the absolute risk
     causes <- object$causes
     index.cause <- which(causes == cause)
     name.model <- names(object$models)
-
+    
     ## event times
     eTimes <- object$eventTimes
 
     ## ** check
     if (any(match(as.character(cause), causes, nomatch = 0)==0L))
-        stop(paste0("Cannot find all requested cause(s) ...\n\n",
-                    "Requested cause(s): ", paste0(cause, collapse = ", "),
-                    "\n Available causes: ", paste(causes, collapse = ", "),
+        stop(paste0("Cannot find all requested cause(s) ...\n\n", 
+                    "Requested cause(s): ", paste0(cause, collapse = ", "), 
+                    "\n Available causes: ", paste(causes, collapse = ", "), 
                     "\n"))
     if (surv.type == "survival") {
-        if (object$theCause != cause)
-            stop("Object can be used to predict cause ", object$theCause,
+        if (object$theCause != cause) 
+            stop("Object can be used to predict cause ", object$theCause, 
                  " but not ", cause, ".\nNote: the cause can be specified in CSC(...,cause=).")
     }
     if(any(is.na(times))){
@@ -199,7 +199,7 @@ predict.CauseSpecificCox <- function(object,
     if(length(landmark)!=1){
         stop("\'t0\' must have length one \n")
     }
-    if(!is.logical(diag)){
+    if(!is.logical(diag)){ 
         stop("Argument \'diag\' must be logical \n")
     }
     type <- match.arg(type, c("absRisk","survival"))
@@ -251,7 +251,9 @@ predict.CauseSpecificCox <- function(object,
     if (is.null(eventTimes)) {
         stop("eventTimes was removed from model - cannot predict")
     }
-
+    eventTimes <- eTimes[eTimes <= max(times)] ## jump times before the last prediction time (that is before the last jump)
+    if(length(eventTimes) == 0){eventTimes <- eTimes[1]} # at least the first event
+    
     ## order prediction times
     otimes <- order(times)
     ootimes <- order(otimes)
@@ -284,31 +286,28 @@ predict.CauseSpecificCox <- function(object,
         ## when surv.type = "hazard" and iterC corresponds to the cause and no se/iid
         ## we could only compute cumhazard (i.e. not compute hazard).
         ## But since computing hazard has little impact on the performance it is done anyway
-        baseline = object$baselines[[iterC]]
-        if (is.null(baseline)) {
-            baseline <- predictCox(object$models[[iterC]],
-                                   centered = FALSE,
-                                   times = eventTimes,
-                                   newdata = NULL,
-                                   type = c("hazard","cumhazard"),
-                                   keep.strata = TRUE,
-                                   keep.times = TRUE,
-                                   se = FALSE,
-                                   keep.infoVar = TRUE)
-        }
+        baseline <- predictCox(object$models[[iterC]],
+                               centered = FALSE,
+                               times = eventTimes,
+                               newdata = NULL,
+                               type = c("hazard","cumhazard"), 
+                               keep.strata = TRUE,
+                               keep.times = TRUE,
+                               se = FALSE,
+                               keep.infoVar = TRUE)
         ls.infoVar[[iterC]] <- baseline$infoVar
 
         ## baseline hazard from the Cox model
         ls.cumhazard[[iterC]] <- matrix(baseline$cumhazard, byrow = FALSE, nrow = nEventTimes)
         ls.hazard[[iterC]] <- matrix(baseline$hazard, byrow = FALSE, nrow = nEventTimes)
-
+          
         ## linear predictor for the new observations
         M.eXb[,iterC] <- exp(coxLP(object$models[[iterC]], data = newdata, center = FALSE))
 
         ## strata for the new observations
-        strataTempo <- coxStrata(object$models[[iterC]], data = newdata,
-                                 sterms = ls.infoVar[[iterC]]$strata.sterms,
-                                 strata.vars = ls.infoVar[[iterC]]$stratavars,
+        strataTempo <- coxStrata(object$models[[iterC]], data = newdata, 
+                                 sterms = ls.infoVar[[iterC]]$strata.sterms, 
+                                 strata.vars = ls.infoVar[[iterC]]$stratavars, 
                                  strata.levels = ls.infoVar[[iterC]]$strata.levels)
         M.strata.num[,iterC] <- as.numeric(strataTempo) - 1
         attr(M.strata.num,paste0("levels",iterC)) <- ls.infoVar[[iterC]]$strata.levels
@@ -320,18 +319,18 @@ predict.CauseSpecificCox <- function(object,
     ## ** compute CIF (aka absolute risk) or event-free survival
     vec.etimes.max <- apply(M.etimes.max,1,max) ## take the max because if not censored for one cause and last event equal to 1 then we have the full curve
     if(type == "absRisk"){
-        outCpp <- predictCIF_cpp(hazard = ls.hazard,
-                                 cumhazard = ls.cumhazard,
-                                 eXb = M.eXb,
+        outCpp <- predictCIF_cpp(hazard = ls.hazard, 
+                                 cumhazard = ls.cumhazard, 
+                                 eXb = M.eXb, 
                                  strata = M.strata.num,
-                                 newtimes = if(diag){times}else{sort(times)},
-                                 etimes = eventTimes,
-                                 etimeMax = vec.etimes.max,
+                                 newtimes = if(diag){times}else{sort(times)}, 
+                                 etimes = eventTimes, 
+                                 etimeMax = vec.etimes.max, 
                                  t0 = landmark,
                                  nEventTimes = nEventTimes,
-                                 nNewTimes = n.times,
+                                 nNewTimes = n.times, 
                                  nData = new.n,
-                                 cause = index.cause - 1,
+                                 cause = index.cause - 1, 
                                  nCause = nCause,
                                  survtype = (surv.type=="survival"),
                                  productLimit = product.limit,
@@ -344,8 +343,8 @@ predict.CauseSpecificCox <- function(object,
         return(.predictSurv_CSC(object, times = times, newdata = newdata, ls.hazard = ls.hazard, eXb = M.eXb,
                                 etimes = eventTimes, etimeMax = vec.etimes.max, strata = M.strata.num,
                                 keep.times = keep.times, keep.strata = keep.strata, keep.newdata = keep.newdata,
-                                se = se, band = band, iid = iid,
-                                confint = confint, diag = diag, average.iid = average.iid,
+                                se = se, band = band, iid = iid, 
+                                confint = confint, diag = diag, average.iid = average.iid, 
                                 store.iid = store.iid, product.limit = product.limit))
     }
 
@@ -363,11 +362,11 @@ predict.CauseSpecificCox <- function(object,
                 new.LPdata[[iCause]] <- model.matrix(object$models[[iCause]], data = newdata)
             }else{
                 new.LPdata[[iCause]] <- matrix(0, ncol = 1, nrow = new.n)
-            }
+            }  
         }
 
         ## linear predictors
-        nVar <- unlist(lapply(ls.infoVar,function(m){
+        nVar.lp <- unlist(lapply(ls.infoVar,function(m){
             length(m$lpvars)
         }))
 
@@ -389,23 +388,23 @@ predict.CauseSpecificCox <- function(object,
                 })
             }
         }
-
+        
         out.seCSC <- calcSeCSC(object,
                                cif = outCpp$cif,
                                hazard = ls.hazard,
                                cumhazard = ls.cumhazard,
                                survival = outCpp$survival, ## survival at t-
                                object.time = eventTimes,
-                               object.maxtime = vec.etimes.max,
+                               object.maxtime = vec.etimes.max, 
                                eXb = M.eXb,
                                new.LPdata = new.LPdata,
-                               new.strata = M.strata.num,
+                               new.strata = M.strata.num,                               
                                times = if(diag){times}else{Utimes},
                                ls.infoVar = ls.infoVar,
                                new.n = new.n,
                                cause = index.cause,
                                nCause = nCause,
-                               nVar = nVar,
+                               nVar.lp = nVar.lp,
                                surv.type = surv.type,
                                export = export,
                                store.iid = store.iid,
@@ -414,7 +413,7 @@ predict.CauseSpecificCox <- function(object,
         ootimes2 <- prodlim::sindex(jump.times = Utimes, eval.times = times)
         needOrder2 <- !identical(1:length(ootimes2),ootimes2)
     }
-
+    
     ## ** export
     ## gather all outputs
     if(needOrder && (diag == FALSE)){
@@ -422,7 +421,7 @@ predict.CauseSpecificCox <- function(object,
     }else{
         out <- list(absRisk = outCpp$cif) # reorder prediction times
     }
-
+    
     if(se+band){
         if(needOrder2 && (diag == FALSE)){
             out$absRisk.se <- out.seCSC$se[,ootimes2,drop=FALSE]
@@ -454,7 +453,7 @@ predict.CauseSpecificCox <- function(object,
     if(keep.times){out$times <- times}
 
     all.covars <- unique(unlist(lapply(ls.infoVar, function(iI){
-        c(iI$lpvars.original, iI$strata.vars.original)
+        c(iI$lpvars.original, iI$stratavars.original)
     })))
     if(keep.newdata[[1]]==TRUE && length(all.covars)>0){
         out$newdata <- newdata[, all.covars, with = FALSE]
@@ -471,7 +470,7 @@ predict.CauseSpecificCox <- function(object,
     out$se <- se
     out$keep.times <- keep.times
     out$band <- band
-
+    
     class(out) <- "predictCSC"
 
     ## compute confidence interval
@@ -500,12 +499,12 @@ predict.CauseSpecificCox <- function(object,
     if(any(etimes<0)){
         stop("Cannot handle negative event times. \n")
     }
-
+        
     new.n <- NROW(newdata)
     n.times <- length(times)
     n.times2 <- ifelse(diag>0,1,n.times)
     nCause <- length(object$cause)
-    # n.sample <- NROW(object$response)
+    n.sample <- NROW(object$response)
 
     iid.save <- iid
     iid <- (iid || se)
@@ -515,7 +514,7 @@ predict.CauseSpecificCox <- function(object,
         }
     }
 
-    ## ** prepare output
+    ## ** prepare output        
     n.sample <- stats::nobs(object)
     out <- list(survival = NULL, survival.iid = NULL, survival.average.iid = NULL,
                 lastEventTime = NA,
@@ -568,7 +567,7 @@ predict.CauseSpecificCox <- function(object,
         }else{
             test.list <- !is.list(factor)
             n.factor <- length(factor)
-
+                    
             for(iFactor in 1:n.factor){ ## iFactor <- 1
                 ## when only one column and diag = FALSE, use the same weights at all times
                 if((diag == FALSE) && (NCOL(factor[[iFactor]])==1) && (NCOL(factor[[iFactor]])==new.n) && (n.times > 1)){
@@ -586,7 +585,7 @@ predict.CauseSpecificCox <- function(object,
             attr(average.iid,"factor") <- factor
         }
     }
-
+    
     ## ** compute iid for survival
     if(iid || se || band || average.iid){
 
@@ -594,7 +593,7 @@ predict.CauseSpecificCox <- function(object,
             iIid <- array(0, dim = c(n.sample, n.times2, new.n))
         }
         if(average.iid){
-
+            
             if(is.null(factor)){
                 iAverageIid <- matrix(0, nrow = n.sample, ncol = n.times2)
             }else{
